@@ -1,4 +1,7 @@
 
+#define BOOST_DISABLE_ASSERTS
+#define ARMA_DONT_PRINT_ERRORS
+
 // [[Rcpp::depends(RcppArmadillo)]]
 // [[Rcpp::plugins(openmp)]]
 // [[Rcpp::depends(BH)]]
@@ -16,7 +19,6 @@ using namespace Rcpp;
 using namespace arma;
 using namespace std;
 
-
 double sqdist(rowvec l1, rowvec l2) { 
   double ssq = 0.0;
   for(arma::uword k = 0; k < l1.size(); ++k) {
